import React, { useState, useCallback, useEffect, createContext, useRef, useReducer, useContext } from 'react';
import useScrollbarSize from 'react-scrollbar-size';
import { motion, AnimatePresence } from 'framer-motion';

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];
      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }
    return target;
  };
  return _extends.apply(this, arguments);
}

var useShortcut = function useShortcut(_ref) {
  var targetKey = _ref.targetKey,
    modifier = _ref.modifier,
    handler = _ref.handler;
  var _useState = useState(false),
    keyPressed = _useState[0],
    setKeyPressed = _useState[1];
  var downHandler = useCallback(function (event) {
    if (event.key === targetKey) {
      if (modifier === 'shift' && event.shiftKey) {
        event.preventDefault();
        setKeyPressed(true);
        handler === null || handler === void 0 ? void 0 : handler();
      } else if (modifier === 'ctrl' && event.ctrlKey) {
        event.preventDefault();
        setKeyPressed(true);
        handler === null || handler === void 0 ? void 0 : handler();
      } else if (modifier === 'alt' && event.altKey) {
        event.preventDefault();
        setKeyPressed(true);
        handler === null || handler === void 0 ? void 0 : handler();
      } else if (modifier === 'meta' && event.metaKey) {
        event.preventDefault();
        setKeyPressed(true);
        handler === null || handler === void 0 ? void 0 : handler();
      } else {
        event.preventDefault();
        setKeyPressed(true);
        handler === null || handler === void 0 ? void 0 : handler();
      }
    }
  }, [handler, setKeyPressed]);
  var upHandler = useCallback(function (event) {
    if (event.key === targetKey) setKeyPressed(false);
  }, [setKeyPressed]);
  useEffect(function () {
    window.addEventListener('keydown', downHandler);
    window.addEventListener('keyup', upHandler);
    return function () {
      window.removeEventListener('keydown', downHandler);
      window.removeEventListener('keyup', upHandler);
    };
  }, [upHandler, downHandler]);
  return keyPressed;
};

var useBodyScrollable = function useBodyScrollable() {
  var _useState = useState(true),
    bodyScrollable = _useState[0],
    setBodyScrollable = _useState[1];
  useEffect(function () {
    var resizeObserver = new ResizeObserver(function () {
      return setBodyScrollable(document.body.scrollHeight > window.innerHeight);
    });
    resizeObserver.observe(document.body);
    return function () {
      return resizeObserver.unobserve(document.body);
    };
  }, []);
  return bodyScrollable;
};

var ActionType;
(function (ActionType) {
  ActionType["INCREASE"] = "INCREASE";
  ActionType["DECREASE"] = "DECREASE";
  ActionType["RESET"] = "RESET";
  ActionType["CUSTOM"] = "CUSTOM";
})(ActionType || (ActionType = {}));

var MenuContext = createContext({});
var initialState = {
  selected: 0
};
var MenuProvider = function MenuProvider(_ref) {
  var children = _ref.children,
    dimensions = _ref.dimensions,
    config = _ref.config;
  var input = useRef(null);
  var _useState = useState(0),
    open = _useState[0],
    setOpen = _useState[1];
  var _useState2 = useState(''),
    query = _useState2[0],
    setQuery = _useState2[1];
  var _useState3 = useState(undefined),
    placeholder = _useState3[0],
    setPlaceholder = _useState3[1];
  var _useState4 = useState(null),
    results = _useState4[0],
    setResults = _useState4[1];
  var _useState5 = useState([]),
    crumbs = _useState5[0],
    setCrumbs = _useState5[1];
  var _useState6 = useState(false),
    animate = _useState6[0],
    setAnimate = _useState6[1];
  var scrollable = useBodyScrollable();
  var _useScrollbarSize = useScrollbarSize(),
    width = _useScrollbarSize.width;
  var reducer = function reducer(state, action) {
    switch (action.type) {
      case ActionType.INCREASE:
        return state.selected === results.index - 1 ? _extends({}, state, {
          selected: results.index - 1
        }) : _extends({}, state, {
          selected: state.selected + 1
        });
      case ActionType.DECREASE:
        return state.selected === 0 ? _extends({}, state, {
          selected: 0
        }) : _extends({}, state, {
          selected: state.selected - 1
        });
      case ActionType.CUSTOM:
        return _extends({}, state, {
          selected: action.custom
        });
      case ActionType.RESET:
        return _extends({}, state, {
          selected: 0
        });
    }
  };
  var _useReducer = useReducer(reducer, initialState),
    state = _useReducer[0],
    dispatch = _useReducer[1];
  var navigation = function navigation(event) {
    if ((event.ctrlKey || event.metaKey) && event.key === 'k') {
      event.preventDefault();
      setQuery('');
      setOpen(1);
    }
    if (event.key === 'Escape' && open === 1) {
      setQuery('');
      setOpen(0);
    }
    if (open > 0) {
      if (event.key === 'Tab' && !event.shiftKey) {
        event.preventDefault();
        dispatch({
          type: ActionType.INCREASE,
          custom: 0
        });
      } else if (event.shiftKey && event.key === 'Tab') {
        event.preventDefault();
        dispatch({
          type: ActionType.DECREASE,
          custom: 0
        });
      }
    }
    if (open > 1 && query === '' && (event.key === 'Backspace' || event.key === 'Escape')) {
      event.preventDefault();
      setAnimate(true);
      setTimeout(function () {
        return setAnimate(false);
      }, 100);
      setQuery('');
      setOpen(1);
    }
  };
  var mobileToggle = function mobileToggle(event) {
    if (event.touches.length >= 2) {
      event.preventDefault();
      setOpen(1);
    }
  };
  useEffect(function () {
    window.addEventListener('keydown', navigation);
    window.addEventListener('touchstart', mobileToggle);
    return function () {
      window.removeEventListener('keydown', navigation);
      window.removeEventListener('touchstart', mobileToggle);
    };
  }, [open, setOpen, query]);
  useEffect(function () {
    dispatch({
      type: ActionType.RESET,
      custom: 0
    });
    setQuery('');
    if (open && scrollable) {
      document.body.style.overflow = 'hidden';
      document.body.style.paddingRight = width + "px";
    } else {
      document.body.style.overflow = 'unset';
      document.body.style.paddingRight = '0px';
    }
  }, [open, setOpen]);
  return React.createElement(MenuContext.Provider, {
    value: {
      query: query,
      setQuery: setQuery,
      placeholder: placeholder,
      setPlaceholder: setPlaceholder,
      results: results,
      setResults: setResults,
      animate: animate,
      setAnimate: setAnimate,
      crumbs: crumbs,
      setCrumbs: setCrumbs,
      open: open,
      setOpen: setOpen,
      config: config,
      dimensions: dimensions,
      state: state,
      dispatch: dispatch,
      input: input
    }
  }, children);
};

var useInView = function useInView(_ref) {
  var ref = _ref.ref;
  var _useState = useState(false),
    isIntersecting = _useState[0],
    setIntersecting = _useState[1];
  var observer = new IntersectionObserver(function (_ref2) {
    var entry = _ref2[0];
    return setIntersecting(entry.isIntersecting);
  });
  useEffect(function () {
    observer.observe(ref.current);
    return function () {
      return observer.disconnect();
    };
  }, []);
  return isIntersecting;
};

var run = function run(command) {
  var _command$perform;
  if (typeof command.perform !== 'undefined') return (_command$perform = command.perform) === null || _command$perform === void 0 ? void 0 : _command$perform.call(command);else if (typeof command.href !== 'undefined') window.open(command.href, command.newTab ? '_blank' : '_self');
};

var Command = function Command(_ref) {
  var onMouseEnter = _ref.onMouseEnter,
    isSelected = _ref.isSelected,
    command = _ref.command;
  var _useContext = useContext(MenuContext),
    setOpen = _useContext.setOpen,
    config = _useContext.config;
  var topRef = useRef(null);
  var bottomRef = useRef(null);
  var enter = useShortcut({
    targetKey: 'Enter'
  });
  var inViewTop = useInView({
    ref: topRef
  });
  var inViewBottom = useInView({
    ref: bottomRef
  });
  useEffect(function () {
    var _bottomRef$current;
    if (isSelected && (!inViewTop || !inViewBottom)) (_bottomRef$current = bottomRef.current) === null || _bottomRef$current === void 0 ? void 0 : _bottomRef$current.scrollIntoView({
      behavior: 'smooth',
      block: 'end'
    });
    if (enter && isSelected) {
      if (!command.closeOnComplete) setOpen(0);
      run(command);
    }
  }, [isSelected, enter]);
  return React.createElement("div", {
    role: 'option',
    "aria-selected": isSelected
  }, React.createElement("span", {
    ref: topRef,
    "aria-hidden": 'true'
  }), command.anchor ? React.createElement(command.anchor, {
    className: 'command',
    onMouseMove: onMouseEnter,
    style: {
      color: isSelected ? (config === null || config === void 0 ? void 0 : config.commandActive) || '#343434' : (config === null || config === void 0 ? void 0 : config.commandInactive) || '#828282'
    },
    onClick: function onClick() {
      if (!command.closeOnComplete) setOpen(0);
      run(command);
    },
    href: command.href || '#',
    target: command.newTab ? '_blank' : '_self',
    rel: 'noreferrer'
  }, isSelected && React.createElement(motion.div, {
    layoutId: 'box',
    className: 'selected',
    initial: false,
    "aria-hidden": 'true',
    transition: {
      type: 'spring',
      stiffness: 1000,
      damping: 80
    },
    style: {
      background: (config === null || config === void 0 ? void 0 : config.barBackground) || '#82828220'
    }
  }), React.createElement("div", {
    className: 'info_wrapper'
  }, command.icon && command.icon, React.createElement("p", {
    className: 'command_text'
  }, command.text)), command.shortcuts && React.createElement("div", {
    className: 'shortcuts'
  }, command.shortcuts.modifier && React.createElement("kbd", {
    style: {
      backgroundColor: (config === null || config === void 0 ? void 0 : config.shortcutBackground) || '#82828220'
    }
  }, command.shortcuts.modifier), command.shortcuts.keys.map(function (key, index) {
    return React.createElement("kbd", {
      key: index,
      style: {
        backgroundColor: (config === null || config === void 0 ? void 0 : config.shortcutBackground) || '#82828220'
      }
    }, key);
  }))) : React.createElement("a", {
    className: 'command',
    onMouseMove: onMouseEnter,
    style: {
      color: isSelected ? (config === null || config === void 0 ? void 0 : config.commandActive) || '#343434' : (config === null || config === void 0 ? void 0 : config.commandInactive) || '#828282'
    },
    onClick: function onClick() {
      if (!command.closeOnComplete) setOpen(0);
      run(command);
    },
    href: command.href || '#',
    target: command.newTab ? '_blank' : '_self',
    rel: 'noreferrer'
  }, isSelected && React.createElement(motion.div, {
    layoutId: 'box',
    className: 'selected',
    initial: false,
    "aria-hidden": 'true',
    transition: {
      type: 'spring',
      stiffness: 1000,
      damping: 80
    },
    style: {
      background: (config === null || config === void 0 ? void 0 : config.barBackground) || '#82828220'
    }
  }), React.createElement("div", {
    className: 'info_wrapper'
  }, command.icon && command.icon, React.createElement("p", {
    className: 'command_text'
  }, command.text)), command.shortcuts && React.createElement("div", {
    className: 'shortcuts'
  }, command.shortcuts.modifier && React.createElement("kbd", {
    style: {
      backgroundColor: (config === null || config === void 0 ? void 0 : config.shortcutBackground) || '#82828220'
    }
  }, command.shortcuts.modifier), command.shortcuts.keys.map(function (key, index) {
    return React.createElement("kbd", {
      key: index,
      style: {
        backgroundColor: (config === null || config === void 0 ? void 0 : config.shortcutBackground) || '#82828220'
      }
    }, key);
  }))), React.createElement("span", {
    ref: bottomRef,
    className: 'scroll_ref',
    "aria-hidden": 'true'
  }));
};

var CommandMenu = function CommandMenu(props) {
  var _useContext = useContext(MenuContext),
    config = _useContext.config,
    results = _useContext.results,
    state = _useContext.state,
    dispatch = _useContext.dispatch;
  return React.createElement(Wrapper, Object.assign({}, props), typeof props.loadingPlaceholder !== 'undefined' && props.loadingState ? props.loadingPlaceholder : results === null || results === void 0 ? void 0 : results.commands.map(function (category, index) {
    return React.createElement("div", {
      key: index
    }, category.commands.length > 0 && React.createElement("p", {
      className: 'category_header',
      style: {
        color: (config === null || config === void 0 ? void 0 : config.headingColor) || '#828282'
      }
    }, category.category), category.commands.map(function (command, index) {
      return React.createElement(Command, {
        onMouseEnter: function onMouseEnter() {
          return dispatch({
            type: ActionType.CUSTOM,
            custom: command.globalIndex
          });
        },
        isSelected: state.selected === command.globalIndex,
        command: command,
        key: index
      });
    }));
  }));
};
var Wrapper = function Wrapper(props) {
  var _useContext2 = useContext(MenuContext),
    open = _useContext2.open,
    query = _useContext2.query,
    setQuery = _useContext2.setQuery,
    setPlaceholder = _useContext2.setPlaceholder,
    results = _useContext2.results,
    setResults = _useContext2.setResults,
    dispatch = _useContext2.dispatch,
    dimensions = _useContext2.dimensions,
    setCrumbs = _useContext2.setCrumbs,
    input = _useContext2.input;
  useEffect(function () {
    if (open === props.index) setPlaceholder(typeof props.placeholder === 'string' ? props.placeholder : 'What do you need?');
  }, [open]);
  useEffect(function () {
    if (open !== props.index) return;
    dispatch({
      type: ActionType.RESET,
      custom: 0
    });
    if (!query || props.preventSearch) {
      if (!query) input.current.value = '';
      setCrumbs(props.crumbs);
      return setResults(props.commands);
    }
    var index = 0;
    var sorted = [];
    props.commands.commands.forEach(function (row) {
      var _row$subCommands;
      console.log(props.commands.commands);
      var results = {
        category: row.category,
        commands: []
      };
      row.commands.forEach(function (command) {
        var _command$keywords;
        var text = command.text.toLowerCase() + ((_command$keywords = command.keywords) === null || _command$keywords === void 0 ? void 0 : _command$keywords.toLowerCase());
        if (text.includes(query.toLowerCase())) {
          results.commands.push(_extends({}, command, {
            globalIndex: index
          }));
          index++;
        }
      });
      (_row$subCommands = row.subCommands) === null || _row$subCommands === void 0 ? void 0 : _row$subCommands.forEach(function (command) {
        var _command$keywords2;
        var text = command.text.toLowerCase() + ((_command$keywords2 = command.keywords) === null || _command$keywords2 === void 0 ? void 0 : _command$keywords2.toLowerCase());
        if (text.includes(query.toLowerCase())) {
          results.commands.push(_extends({}, command, {
            globalIndex: index
          }));
          index++;
        }
      });
      if (results.commands.length > 0) sorted.push(results);
    });
    return setResults({
      index: index,
      commands: sorted,
      initialHeight: props.commands.initialHeight
    });
  }, [query, setQuery, open, props.loadingState]);
  var up = useShortcut({
    targetKey: 'ArrowUp'
  });
  var down = useShortcut({
    targetKey: 'ArrowDown'
  });
  useEffect(function () {
    if (open === props.index) {
      if (up) dispatch({
        type: ActionType.DECREASE,
        custom: 0
      });else if (down) dispatch({
        type: ActionType.INCREASE,
        custom: 0
      });
    }
  }, [up, down]);
  if (open !== props.index || typeof (results === null || results === void 0 ? void 0 : results.index) === 'undefined') return null;
  return React.createElement(motion.div, {
    className: 'command_wrapper',
    role: 'listbox',
    style: {
      overflowY: results.index >= 5 ? 'auto' : 'hidden',
      height: results.index >= 5 ? results === null || results === void 0 ? void 0 : results.initialHeight : props.loadingState ? 'auto' : results.commands.length * ((dimensions === null || dimensions === void 0 ? void 0 : dimensions.sectionHeight) || 31) + results.index * ((dimensions === null || dimensions === void 0 ? void 0 : dimensions.commandHeight) || 54)
    }
  }, props.children);
};

var useClickOutside = function useClickOutside(_ref) {
  var ref = _ref.ref,
    handler = _ref.handler;
  useEffect(function () {
    var listener = function listener(event) {
      if (!ref.current || ref.current.contains(event.target)) return;
      if (handler) handler();
    };
    document.addEventListener('mousedown', listener);
    document.addEventListener('touchstart', listener);
    return function () {
      document.removeEventListener('mousedown', listener);
      document.removeEventListener('touchstart', listener);
    };
  }, [ref, handler]);
};

var CommandWrapper = function CommandWrapper(_ref) {
  var _config$animationDura;
  var children = _ref.children,
    value = _ref.value;
  var menuRef = useRef(null);
  var _useContext = useContext(MenuContext),
    open = _useContext.open,
    setOpen = _useContext.setOpen,
    placeholder = _useContext.placeholder,
    animate = _useContext.animate,
    config = _useContext.config,
    setQuery = _useContext.setQuery,
    state = _useContext.state,
    crumbs = _useContext.crumbs,
    input = _useContext.input;
  useClickOutside({
    ref: menuRef,
    handler: function handler() {
      return setOpen(0);
    }
  });
  return React.createElement(AnimatePresence, null, open > 0 && React.createElement(motion.div, {
    className: 'backdrop',
    initial: {
      opacity: 0,
      pointerEvents: 'none'
    },
    animate: {
      opacity: 1,
      pointerEvents: 'auto'
    },
    exit: {
      opacity: 0,
      pointerEvents: 'none'
    },
    style: {
      backgroundColor: (config === null || config === void 0 ? void 0 : config.backdropColor) || '#FFFFFF90',
      backdropFilter: config !== null && config !== void 0 && config.backdropBlur ? "blur(" + (config === null || config === void 0 ? void 0 : config.backdropBlur) + "px)" : 'blur(2px)'
    }
  }, React.createElement(motion.div, {
    className: 'dialog',
    role: 'dialog',
    "aria-modal": 'true',
    ref: menuRef,
    initial: {
      opacity: 0,
      scale: 0.95
    },
    animate: {
      opacity: 1,
      scale: animate ? 0.98 : 1
    },
    exit: {
      opacity: 0,
      scale: 0.95
    },
    transition: {
      duration: (_config$animationDura = config === null || config === void 0 ? void 0 : config.animationDuration) != null ? _config$animationDura : 0.1
    },
    style: {
      backgroundColor: (config === null || config === void 0 ? void 0 : config.backgroundColor) || '#FFFFFF',
      borderColor: (config === null || config === void 0 ? void 0 : config.borderColor) || 'transparent',
      borderWidth: (config === null || config === void 0 ? void 0 : config.borderWidth) || 0,
      borderRadius: (config === null || config === void 0 ? void 0 : config.borderRadius) || '10px',
      boxShadow: (config === null || config === void 0 ? void 0 : config.boxShadow) || '0px 0px 60px 10px #00000020'
    }
  }, React.createElement("div", {
    className: 'crumbs'
  }, crumbs === null || crumbs === void 0 ? void 0 : crumbs.map(function (crumb, index) {
    return React.createElement("button", {
      onClick: function onClick() {
        return setOpen(index + 1);
      },
      className: 'breadcrumb',
      style: {
        backgroundColor: (config === null || config === void 0 ? void 0 : config.breadcrumbColor) || '#EFEFEF',
        borderRadius: (config === null || config === void 0 ? void 0 : config.breadcrumbRadius) || 5
      },
      key: index
    }, crumb);
  })), React.createElement("input", {
    placeholder: placeholder || 'What do you need?',
    defaultValue: value,
    className: 'searchbar',
    "aria-expanded": 'true',
    "aria-autocomplete": 'list',
    "aria-haspopup": 'listbox',
    "aria-readonly": 'true',
    role: 'combobox',
    autoFocus: true,
    spellCheck: 'false',
    ref: input,
    "aria-activedescendant": state.selected.toString(),
    onChange: function onChange(e) {
      return setQuery(e.target.value);
    },
    style: {
      color: (config === null || config === void 0 ? void 0 : config.inputColor) || '#000000',
      borderBottom: ((config === null || config === void 0 ? void 0 : config.inputBorder) || '#e9ecef') + " 1px solid"
    }
  }), children)));
};

var useCommands = function useCommands(initialCommands) {
  var _useContext = useContext(MenuContext),
    dimensions = _useContext.dimensions;
  var _useState = useState(),
    height = _useState[0],
    setHeight = _useState[1];
  var _useState2 = useState(),
    index = _useState2[0],
    setIndex = _useState2[1];
  var _useState3 = useState(function () {
      var currentCategories = 0;
      var height = 0;
      var index = 0;
      var sorted = [];
      initialCommands === null || initialCommands === void 0 ? void 0 : initialCommands.forEach(function (category) {
        var _category$subCommands;
        currentCategories++;
        var indexedCommands = category.commands.map(function (command) {
          index++;
          if (index <= 5) height = currentCategories * ((dimensions === null || dimensions === void 0 ? void 0 : dimensions.sectionHeight) || 31) + index * ((dimensions === null || dimensions === void 0 ? void 0 : dimensions.commandHeight) || 54);
          return _extends({}, command, {
            globalIndex: index - 1
          });
        });
        var subCommands = (_category$subCommands = category.subCommands) === null || _category$subCommands === void 0 ? void 0 : _category$subCommands.map(function (command) {
          return _extends({}, command, {
            globalIndex: 0
          });
        });
        sorted.push({
          category: category.category,
          commands: indexedCommands,
          subCommands: subCommands
        });
      });
      setHeight(height);
      setIndex(index);
      return sorted;
    }),
    commands = _useState3[0],
    setCommands = _useState3[1];
  return [{
    index: index,
    commands: commands,
    initialHeight: height
  }, useCallback(function (cmds) {
    var currentCategories = 0;
    var height = 0;
    var index = 0;
    setCommands(function () {
      var sorted = [];
      cmds.forEach(function (category) {
        currentCategories++;
        var indexedCommands = category.commands.map(function (command) {
          index++;
          if (index <= 5) height = currentCategories * 31 + index * 54;
          return _extends({}, command, {
            globalIndex: index - 1
          });
        });
        sorted.push({
          category: category.category,
          commands: indexedCommands
        });
      });
      setHeight(height);
      setIndex(index);
      return sorted;
    });
  }, [commands])];
};

var useKmenu = function useKmenu() {
  var context = useContext(MenuContext);
  if (!context) throw new Error('useKmenu must be called inside the MenuProvider');
  var toggle = useCallback(function () {
    context.setOpen(function (open) {
      return open === 0 ? 1 : 0;
    });
  }, []);
  var openNestedMenu = useCallback(function (index) {
    context.setAnimate(true);
    setTimeout(function () {
      return context.setAnimate(false);
    }, 100);
    context.setOpen(index);
  }, []);
  return {
    input: context.query,
    setInput: context.setQuery,
    open: context.open,
    setOpen: openNestedMenu,
    toggle: toggle
  };
};

export { CommandMenu, CommandWrapper, MenuProvider, useCommands, useKmenu, useShortcut };
//# sourceMappingURL=index.modern.js.map
